/**
 * apeman app to handle sign.
 * @function apemanAppSign
 * @param {Object} UserSign - UserSignModel.
 * @param {Object} [options] - Optional settings.
 * @param {Object} [options.signCreateSchema] - Schema for sign create.
 * @param {Object} [options.signUpdateSchema] - Schema for sign update.
 * @param {Object} [options.signPassCreateSchema] - Schema for sign pass create.
 * @param {string} [options.format] - Response format. "json", "xml", etc.
 * @param {number} [options.saltLength=24] - Length of salt.
 * @param {number} [options.passwordSpec='pdkdf2:120:84'] - Password algorithm, iterations, length.
 * @returns {function} - Defined app function.
 */

'use strict'

const argx = require('argx')
const rest = require('apeman-app-rest')
const objnest = require('objnest')
const assert = require('assert')
const defineDigest = require('./digesting/define_digest')
const schemas = require('./schemas')

/** @lends create */
function create (UserSign, options) {
  let args = argx(arguments)
  options = objnest.expand(args.pop('object') || {})

  assert.ok(UserSign, 'UserSign is required.')
  assert.ok(!!(UserSign.$$inherited && UserSign.$$inherited.ApUserSign), 'UserSign should inherit ApUserSign.')

  let pathname = options.pathname || '/sign'
  let format = options.format
  let digest = defineDigest(options.passwordSpec)
  return [].concat(
    rest(UserSign, {
      format,
      pathname: `${pathname}/pass`,
      createSchema: options.signPassCreateSchema || schemas('sign-pass-create'),
      updateDisabled: true,
      bulkUpdateDisabled: true,
      doCreate: require('./doing/do_create_sign_pass')({ digest }),
      doOne: require('./doing/do_one_sign_pass')({}),
      doList: require('./doing/do_list_sign_pass')({}),
      doDestroy: require('./doing/do_destroy_sign_pass')({}),
      doBulkDestroy: require('./doing/do_destroy_sign_pass')({}).bulk
    }),
    rest(UserSign, {
      format,
      pathname,
      doCreate: require('./doing/do_create_sign')({ digest }),
      doUpdate: require('./doing/do_update_sign')({ digest }),
      doBulkUpdate: require('./doing/do_update_sign')({ digest }).bulk,
      doDestroy: require('./doing/do_destroy_sign')({}),
      doBulkDestroy: require('./doing/do_destroy_sign')({}).bulk,
      createSchema: options.signCreateSchema || schemas('sign-create'),
      updateSchema: options.signUpdateSchema || schemas('sign-update')
    })
  )
}

module.exports = create
